"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useGlobal = void 0;
var React = require("react");
var ts_functional_1 = require("ts-functional");
// Record all components who are subscribed to specific global states
var subscribers = {};
var getSubscribers = function (index) {
    if (!subscribers[index]) {
        subscribers[index] = new Set();
    }
    return subscribers[index];
};
// Record all spies that are listening to state changes
var spies = {};
var globalSpies = new Set();
// Record the last values for all state.  We need this in case the index for a hook changes,
// and we need to manually reset the state
var curValues = {};
var updateSubscribers = function (index, onUpdate) { return function (newValOrSetter) {
    // Convert the new value into a setter function if it's a raw value
    var updateVal = typeof newValOrSetter === 'function'
        ? newValOrSetter
        : function () { return newValOrSetter; };
    // We need to wrap the setter function before sending it to the subscribers, so 
    // that the value can updated (if needed) after it's generated.  The wrapper function is
    // memoized, so that the new value is only calculated and updated once.
    var newSetter = ts_functional_1.memoize(function (old) {
        var newVal = updateVal(old);
        curValues[index] = newVal;
        (spies[index] || []).forEach(function (spy) { spy(old, newVal); });
        globalSpies.forEach(function (spy) { spy(old, newVal, index); });
        if (onUpdate) {
            onUpdate(index, newVal);
        }
        return newVal;
    }, {});
    // Update the subscribers if there are any
    getSubscribers(index).forEach(function (setter) { setter(newSetter); });
}; };
var manageSubscribers = function (index, get, setVal) {
    React.useEffect(function () {
        var subs = getSubscribers(index);
        subs.add(setVal);
        if (curValues[index]) {
            setVal(curValues[index]);
        }
        return function () {
            subs.delete(setVal);
        };
    }, [index, get, setVal]);
};
var useGlobalRaw = function (options) {
    return function (index, initialValue) {
        var get = !!options && !!options.loadInitialValue
            ? function () { return options.loadInitialValue(index, initialValue); }
            : function () { return initialValue; };
        var _a = __read(React.useState(index), 2), curIndex = _a[0], setCurIndex = _a[1];
        var _b = __read(React.useState(get()), 2), val = _b[0], setVal = _b[1];
        manageSubscribers(index, get, setVal);
        var set = updateSubscribers(index, options === null || options === void 0 ? void 0 : options.onUpdate);
        // If the index of this hook changes, we need to manually update the current state based on the last saved value
        React.useEffect(function () {
            if (curIndex !== index) {
                setCurIndex(index);
                set(curValues[index] || get());
            }
        }, [index]);
        return [val, set];
    };
};
useGlobalRaw.listen = {
    on: function (index, spy) {
        if (!spies[index]) {
            spies[index] = new Set();
        }
        spies[index].add(spy);
    },
    onAll: function (spy) {
        globalSpies.add(spy);
    },
    off: function (index, spy) {
        spies[index].delete(spy);
    },
    offAll: function (spy) {
        globalSpies.delete(spy);
    },
    clear: function (index) {
        spies[index].clear();
    },
    clearAll: function () {
        Object.keys(spies).forEach(useGlobalRaw.listen.clear);
        globalSpies.clear();
    }
};
useGlobalRaw.clear = function (index) {
    delete curValues[index];
};
useGlobalRaw.clearAll = function () {
    Object.keys(curValues).forEach(useGlobalRaw.clear);
};
exports.useGlobal = useGlobalRaw;
