"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLocalStorage = void 0;
var useGlobal_1 = require("./useGlobal");
var loadLocalStorageValue = function (deserialize, serialize) { return function (index, initialValue) {
    var localVal = window.localStorage.getItem(index);
    var parsedVal;
    if (localVal === null) {
        window.localStorage.setItem(index, serialize(initialValue));
        parsedVal = initialValue;
    }
    else {
        try {
            parsedVal = deserialize(localVal);
        }
        catch (e) {
            parsedVal = initialValue;
        }
    }
    return parsedVal;
}; };
var saveLocalStorageValue = function (serialize) { return function (index, newValue) {
    window.localStorage.setItem(index, serialize(newValue));
}; };
var useLocalStorageRaw = function (options) { return useGlobal_1.useGlobal({
    loadInitialValue: loadLocalStorageValue(options.deserialize, options.serialize),
    onUpdate: saveLocalStorageValue(options.serialize),
}); };
useLocalStorageRaw.string = function (index, i) {
    return useLocalStorageRaw({ deserialize: function (a) { return a; }, serialize: function (a) { return a; } })(index, i);
};
useLocalStorageRaw.number = function (index, i) {
    return useLocalStorageRaw({ deserialize: function (a) { return +a; }, serialize: function (a) { return "" + a; } })(index, i);
};
useLocalStorageRaw.boolean = function (index, i) {
    return useLocalStorageRaw({ deserialize: function (a) { return !!a; }, serialize: function (a) { return a ? "1" : ""; } })(index, i);
};
useLocalStorageRaw.object = function (index, i) {
    return useLocalStorageRaw({ deserialize: JSON.parse, serialize: JSON.stringify })(index, i);
};
exports.useLocalStorage = useLocalStorageRaw;
